/*-
 * Copyright (c) 2007-2008 Michael Telahun Makonnen <mtm@FreeBSD.Org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#ifndef _CONF_H_
#define _CONF_H_

#include <sys/queue.h>

#include <stdio.h>

/*
 * Strings denoting various xml tags/attributes. Code should use these
 * constants instead of hard-coding the strings. Define in alphabatical order.
 *
 * The following are for fin-config.xml
 */
#define TAG_DISTID			"distid"
#define TAG_HOSTNAME			"hostname"
#define TAG_ROOTPASSWORD		"root_password"

/*
 * fin-backend.xml
 */
#define ATTR_ARCH			"arch"
#define ATTR_CONTAINER			"container"
#define ATTR_DST			"dst"
#define ATTR_FILE			"file"
#define ATTR_ID				"id"
#define ATTR_REQUIRED			"required"
#define ATTR_SRC			"src"
#define TAG_DESC			"desc"
#define TAG_DIST			"dist"
#define TAG_DISTRIBUTIONS		"distributions"

/*
 * Used to specify whether backend or config xml handlers should be
 * used.
 */
enum xhandler_t {XT_BE, XT_CONFIG};

/*
 * Structures related to backend xml configuration.
 */
TAILQ_HEAD(be_distq, be_dist);
struct be_dist {
	TAILQ_ENTRY(be_dist) bd_link;		/* next distribution */
	struct be_distq bd_distq;		/* queue of subdistributions */
	char *bd_file;				/* filename and directory */
	char *bd_desc;				/* short description line */
	char *bd_id;				/* unique id */
	char *bd_src;				/* src directory on media */
	char *bd_dst;				/* dest. install directory */
	char *bd_arch;				/* only for archs... */
	int   bd_required;			/* required distribution */
};

struct be_conf {
	struct be_distq bc_distq;		/* available distributions */
};

/*
 * Structures related to xml installation configuration.
 */
TAILQ_HEAD(cf_dptrq, cf_dptr);
struct cf_dptr {
	TAILQ_ENTRY(cf_dptr) cd_link;		/* next selector */
	char *cd_id;				/* id of selected dist. */
};

struct cf_conf {
	struct cf_dptrq cc_dptrq;		/* q of distribution ids */
	char *cc_hostname;			/* name of installed machine */
	char *cc_rootpw;			/* root's cleartext password */
};

/*
 * Passed in to xml2mem() function so that it can distinguish
 * between the backend xml configuration and installation xml configuration.
 */
union conf_selector {
	struct be_conf cs_be;
	struct cf_conf cs_cf;
};

void	destroy_bedist(struct be_dist *);
void	destroy_dptr(struct cf_dptr *);
void	new_bedist(struct be_dist **, size_t);
void	new_dptr(struct  cf_dptr **, size_t);
int	xml2mem(FILE *, union conf_selector *, enum xhandler_t);
#endif /* _CONF_H_ */
